library WebpInfo;

{$IFDEF FPC}
  {$MODE delphiunicode}
{$ENDIF}

{$R *.res}
{$IFDEF WIN64}
  {$EXTENSION 'wdx64'}
{$ENDIF}
{$IFDEF WIN32}
  {$EXTENSION 'wdx'}
{$ENDIF}
{$SetPEOptFlags $160}

uses
  SysUtils,
  Windows,
  Classes,
  contplug,
  RTLConsts,
  WEBPFormat;

const
  cFieldsNames: array [0..11] of AnsiString = ('Width', 'Height', 'Encoding',
    'Extended Format', 'Alpha', 'ICC', 'EXIF', 'XMP', 'Animation', 'Frame Count',
    'Duration', 'Bit Depth');

  cFieldsTypes: array [0..High(cFieldsNames)] of Integer =
    (ft_Numeric32, ft_Numeric32, ft_MultipleChoice,
    ft_Boolean, ft_Boolean, ft_Boolean, ft_Boolean, ft_Boolean, ft_Boolean,
    ft_Numeric32, ft_Numeric32, ft_Numeric32);

  cWebpEncodingString: array [0..2] of AnsiString = ('Unknown', 'Lossless', 'Lossy');

  DetectStr = 'EXT="WEBP"';

type
  TLastInfo = record
    FileName: string;
    WebpInfo: TWEBPInfo;
  end;

var
  RTL: TRTLCriticalSection;
  LastInfo: TLastInfo;


function FileOpenAsReadOnly(const sFileName: UnicodeString): THandle;
begin
  Result := CreateFileW(PWideChar(sFileName), GENERIC_READ, FILE_SHARE_READ or
    FILE_SHARE_WRITE or FILE_SHARE_DELETE, nil, OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL, 0);
  if Result = INVALID_HANDLE_VALUE then
    raise EFOpenError.CreateResFmt(@SFOpenErrorEx, [ExpandFileName(sFileName),
      SysErrorMessage(GetLastError)])
end;


function ProcessFile(const FileName: UnicodeString; FieldIndex, UnitIndex: Integer): Boolean;
var
  FS: THandleStream;
begin
  Result := LastInfo.WebpInfo.Valid;
  try
    if LastInfo.FileName <> FileName then
    begin
      FS := THandleStream.Create(FileOpenAsReadOnly(FileName));
      try
        LastInfo := Default(TLastInfo);
        LastInfo.FileName := FileName;
        Result := ReadWEBPHeader(FS, LastInfo.WEBPInfo);
      finally
        FileClose(FS.Handle);
        FS.Free;
      end;
    end;
  except
    Result := False;
  end;
end;

function ContentGetSupportedField(FieldIndex: Integer; FieldName: PAnsiChar;
  Units: PAnsiChar; MaxLen: Integer): Integer; stdcall;
var
  Str: AnsiString;
  I: Integer;
begin
  if FieldIndex in [0..High(cFieldsNames)] then
  begin
    StrCopy(Units,'');
    StrLCopy(FieldName, PAnsiChar(cFieldsNames[FieldIndex]), MaxLen - 1);
    Result := cFieldsTypes[FieldIndex];
    StrLCopy(Units, PAnsiChar(''), MaxLen - 1);
    for I := Low(cWebpEncodingString) to High(cWebpEncodingString) do
      Str := cWebpEncodingString[I] + '|';
    if FieldIndex = 2 then
      StrLCopy(Units, PAnsiChar(Str), MaxLen - 1);
  end else
    Result := ft_NoMoreFields;
end;

function GetValueW(FileName: PWideChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; MaxLen, Flags: Integer): Integer;
var
  AFileName: string;
begin
  Result := ft_FieldEmpty;

  AFileName := FileName;
  if Length(AFileName) > 256 then
    if Pos('\\', AFileName) > 0 then
      AFileName := '\\?\UNC\' + Copy(AFileName, 3, Length(AFileName) - 2)
    else
      AFileName := '\\?\' + AFileName;

  if not FileExists(AFileName) then
    Exit;

  try
    if not ProcessFile(AFileName, FieldIndex, UnitIndex) then
      Exit;
  except
    Exit;
  end;

  case FieldIndex of
    0: PInteger(FieldValue)^ := LastInfo.WebpInfo.Width;
    1: PInteger(FieldValue)^ := LastInfo.WebpInfo.Height;
    2: StrLCopy(PAnsiChar(FieldValue), PAnsiChar(cWebpEncodingString[Ord(LastInfo.WebpInfo.Compression)]), MaxLen div 2 - 2);
    3: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.Extended);
    4: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.Alpha);
    5: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.ICC);
    6: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.Exif);
    7: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.XMP);
    8: PInteger(FieldValue)^ := Integer(LastInfo.WebpInfo.Animation);
    9: PInteger(FieldValue)^ := LastInfo.WebpInfo.FrameCount;
    10: PInteger(FieldValue)^ := LastInfo.WebpInfo.Duration;
    11: PInteger(FieldValue)^ := LastInfo.WebpInfo.BitDepth;
  end;

  if FieldIndex in [0 .. High(cFieldsNames)] then
    Result := cFieldsTypes[FieldIndex]
  else
    Result := ft_fieldempty;
end;

function ContentGetValueW(FileName: PWideChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; MaxLen, Flags: Integer): Integer; stdcall;
begin
  if (Flags and CONTENT_DELAYIFSLOW > 0) then
  begin
    Result := FT_DELAYED;
    Exit;
  end;

  EnterCriticalSection(RTL);
  try
    Result := GetValueW(FileName, FieldIndex, UnitIndex, FieldValue, MaxLen, Flags);
  finally
    LeaveCriticalSection(RTL);
  end;
end;

function ContentGetValue(FileName: PAnsiChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; MaxLen, Flags: Integer): Integer; stdcall;
begin
  Result := ContentGetValueW(PWideChar(WideString(AnsiString(FileName))),
    FieldIndex, UnitIndex, FieldValue, MaxLen, Flags)
end;

procedure ContentGetDetectString(DetectString: PAnsiChar; MaxLen: integer); stdcall;
begin
  StrLCopy(DetectString, PAnsiChar(DetectStr), MaxLen - 1);
end;

procedure ContentSetDefaultParams(dps: PContentDefaultParamStruct); stdcall;
begin
end;

procedure DLLEntryProc;
begin
  DeleteCriticalSection(RTL);
end;

exports
  ContentGetDetectString,
  ContentGetSupportedField,
  ContentGetValue,
  ContentGetValueW;
  //ContentSetDefaultParams;

begin
  InitializeCriticalSectionAndSpinCount(RTL, 4000);
  DisableThreadLibraryCalls(HInstance);
  Dll_Process_Detach_Hook := @DLLEntryProc;
end.
